"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BridgeModel = void 0;
const mongoose_1 = require("mongoose");
const aggregate_1 = require("./aggregate");
class BridgeModel {
    modelName;
    mongooseModel;
    modelInterface;
    DBI;
    constructor(schema, modelName) {
        this.modelName = modelName;
        const mongooseSchema = new mongoose_1.Schema(schema.schemaDef, schema.config || { timestamps: false });
        this.mongooseModel = (0, mongoose_1.model)(modelName, mongooseSchema);
    }
    aggregate = () => new aggregate_1.Aggregate(this.mongooseModel);
    create = async (data) => {
        try {
            return (await this.mongooseModel.create([data]))[0].toJSON();
        }
        catch (err) {
            if (err.code === 11000)
                return {
                    error: { status: 409, name: `${this.modelName} already exists`, data: err.keyValue },
                };
            // Should we handle some other errors here ?
            else
                throw err;
        }
    };
    find = async (filter, projection, options) => {
        try {
            return await this.mongooseModel.find(filter, projection || undefined, {
                lean: true,
                ...(options || {}),
            });
        }
        catch (err) {
            // Should we handle some errors here ?
            throw err;
        }
    };
    findById = async (filter, projection, options) => {
        try {
            const res = await this.mongooseModel.findById(filter, projection || undefined, {
                lean: true,
                ...(options || {}),
            });
            if (!res)
                return { error: { status: 404, name: `${this.modelName} not found` } };
            return res;
        }
        catch (err) {
            // Should we handle some errors here ?
            throw err;
        }
    };
    findOne = async (filter, projection, options) => {
        try {
            const res = await this.mongooseModel.findOne(filter, projection || undefined, {
                lean: true,
                ...(options || {}),
            });
            if (!res)
                return { error: { status: 404, name: `${this.modelName} not found` } };
            return res;
        }
        catch (err) {
            // Should we handle some errors here ?
            throw err;
        }
    };
    findOneAndUpdate = async (filter, updateQuery, options) => {
        try {
            const res = await this.mongooseModel.findOneAndUpdate(filter, updateQuery, {
                lean: true,
                ...options,
            });
            if (!res)
                return { error: { status: 404, name: `${this.modelName} not found` } };
            return res;
        }
        catch (err) {
            // Should we handle some errors here ?
            throw err;
        }
    };
    findByIdAndUpdate = async (filter, updateQuery, options) => {
        try {
            const res = await this.mongooseModel.findByIdAndUpdate(filter, updateQuery, {
                lean: true,
                ...options,
            });
            if (!res)
                return { error: { status: 404, name: `${this.modelName} not found` } };
            return res;
        }
        catch (err) {
            // Should we handle some errors here ?
            throw err;
        }
    };
    findOneAndDelete = async (filter, options) => {
        try {
            const res = await this.mongooseModel.findOneAndDelete(filter, options);
            if (!res)
                return { error: { status: 404, name: `${this.modelName} not found` } };
            return res;
        }
        catch (err) {
            // Should we handle some errors here ?
            throw err;
        }
    };
    findByIdAndDelete = async (filter, options) => {
        try {
            const res = await this.mongooseModel.findByIdAndDelete(filter, options);
            if (!res)
                return { error: { status: 404, name: `${this.modelName} not found` } };
            return res;
        }
        catch (err) {
            // Should we handle some errors here ?
            throw err;
        }
    };
    exists = async (filter) => {
        try {
            return (await this.mongooseModel.exists(filter)) !== null;
        }
        catch (err) {
            // Should we handle some errors here ?
            throw err;
        }
    };
    countDocuments = async (filter) => {
        try {
            return await this.mongooseModel.countDocuments(filter);
        }
        catch (err) {
            // Should we handle some errors here ?
            throw err;
        }
    };
}
exports.BridgeModel = BridgeModel;
__exportStar(require("./types"), exports);
